clc;
close all;
clear all;

% This example demonstrates how to implement interleaved tracks for
% subjective comparison experiments (e.g., comparing which of the two
% sounds is louder). For details about the interleaved tracks, see:
%
% Jesteadt, W. (1980). An adaptive procedure for subjective judgments. 
% Attention, Perception, & Psychophysics, 28(1), 85-88.

% psychometric function parameters for the similated observer
phi0 = [4,0.8,0.05,0.05];

% intialize the upper-percentage track (3-down, 1-up)
umlH = UML(exp_config_logit());
umlH.setPhi0(phi0);
umlH.setX0(20);

% initialize the lower-percentage track (1-down, 3-up)
umlL = UML(exp_config_logit());
umlL.setPhi0(phi0);
umlL.setX0(-20);
umlL.par.ndown = 1; % use the same configuration file to initiate but change the ndown and mup variable values
umlL.par.mup = 3;

nrevs = 12;  % the number of reversals to terminate the track.
ntrials = 0;   % total number of 

while umlH.nrev < nrevs &&  umlL.nrev < nrevs % run a pair of trials if at least one track has not yet reached the preset total number of reversals (nrevs).
    ntrials = ntrials + 1;
    
    % simulate an response based on umlH.xnext.
    rH = umlH.simulateResponse(umlH.xnext);
    
    % update the posterior parameter distribution in the low track. This
    % does not change the value in umlL.xnext.
    umlL.update_posterior(umlH.xnext, rH);
    
    % update the high track, which will also compute the new umlH.xnext.
    umlH.update(rH);
    
    % simulate an response based on umlL.xnext.
    rL = umlL.simulateResponse(umlL.xnext);
    
    % update the posterior parameter distribution in the high track. This
    % does not change the value in umlH.xnext.
    umlH.update_posterior(umlL.xnext, rL);
    
    % update the low track, which will also compute the new umlL.xnext.
    umlL.update(rL);
end

figure;
subplot(321);
plot(umlH.phi(:,1),'k','LineWidth',2);hold;
line([0 ntrials],phi0(1)*ones(1,2),'Color','k','LineStyle','--');
ylabel('\alpha');
axis([0 ntrials umlH.par.alpha.limits+[-3 3]]);
set(gca,'XTickLabel',[]);
title('High-Track');

subplot(323);
semilogy(umlH.phi(:,2),'k','LineWidth',2);hold;
line([0 ntrials],phi0(2)*ones(1,2),'Color','k','LineStyle','--');
ylabel('\beta');
axis([0 ntrials umlH.par.beta.limits.*[0.5 2]]);
set(gca,'XTickLabel',[],'YTick',[0.1 1 10]);

subplot(325);
plot(umlH.swpts);hold;
plot(umlH.x,'k','linewidth',2);
axis([0 ntrials umlH.par.alpha.limits+[-3 3]]);
xlabel('Trial Number');
ylabel('Signal Strength');

subplot(322);
plot(umlL.phi(:,1),'k','LineWidth',2);hold;
line([0 ntrials],phi0(1)*ones(1,2),'Color','k','LineStyle','--');
ylabel('\alpha');
axis([0 ntrials umlL.par.alpha.limits+[-3 3]]);
set(gca,'XTickLabel',[]);
title('Low-Track');

subplot(324);
semilogy(umlL.phi(:,2),'k','LineWidth',2);hold;
line([0 ntrials],phi0(2)*ones(1,2),'Color','k','LineStyle','--');
ylabel('\beta');
axis([0 ntrials umlL.par.beta.limits.*[0.5 2]]);
set(gca,'XTickLabel',[],'YTick',[0.1 1 10]);

subplot(326);
plot(umlL.swpts);hold;
plot(umlL.x,'k','linewidth',2);
axis([0 ntrials umlL.par.alpha.limits+[-3 3]]);
xlabel('Trial Number');
ylabel('Signal Strength');
